package ml;

import ml.active.LearnersFactory;
import ml.active.XActiveLearner;
import ml.active.XActiveEvaluationPlotter;
import ml.util.config.XmlReadOnlyConfig;

/**
 * <p>Title: Active Learning</p>
 *
 * <p>Description: Conducts an Active Learning experiment</p>
 *
 * <p>Copyright: Copyright (c) 2005</p>
 *
 * <p>Company: </p>
 *
 * @author not attributable
 * @version 1.0
 */
public class Experimenter {
    private static final String H_DELIM = "----------------------------";

    public Experimenter() {
    }

    public void run(String configFileName) {
        try {
            XmlReadOnlyConfig config = new XmlReadOnlyConfig();
            config.init(configFileName);

            String[] run_args = {"-t", config.get("run/dataset"),
                                "-o", config.get("run/resFile"),
                                "-x", config.get("run/folds"),
                                "-m", config.get("run/num_query")};

            LearnersFactory factory = new LearnersFactory();

            XActiveLearner learner = factory.create(config.get("run/alg"),
                    config);

            System.out.println(XActiveEvaluationPlotter.evaluateModelAndPlot(
                    learner, run_args));

        } catch (Exception exp) {
            System.out.println("");
            System.out.println(H_DELIM);
            System.out.println("Experimenter - Failure \n\n");
            exp.printStackTrace();
            System.out.println(H_DELIM);
        } catch (Error err) {
            System.out.println("");
            System.out.println(H_DELIM);
            System.out.println("Experimenter - Error \n\n");
            System.out.println(err.toString());
            System.out.println(H_DELIM);
        }
    }

    private static void printHelp(){
        System.out.println("Experimenter Help: ");
        System.out.println("------------------");
        System.out.println("Usage: \nml.Experimenter");
        System.out.println("Info: \nfor configuration see/edit config/config.xml");
        System.out.println("the running will produce two files under the 'res' folder:");
        System.out.println("\t1) a matlab file which produce the corresponding learning curve;\n\t 2) a txt file with a summary.");
        System.out.println("(!) for more info regarding running setup consult config/config.xml");
    }

    public static void main(String []args) {
        if (args.length > 0) {
            printHelp();
        }
        else {
            Experimenter experimenter = new Experimenter();
            experimenter.run("config/config.xml"); //hard codded config name
        }
    }
}
