package ml.util.xml;

import org.dom4j.Document;
import org.dom4j.DocumentException;

import org.dom4j.Node;
import org.dom4j.io.SAXReader;

import java.util.*;

/**
 * Reads elements out of an XML files according to XPath routes.
 *
 */
public class XPathReader {

  private Document xmlDoc;

  public XPathReader(String xmlDocName){
    SAXReader reader = new SAXReader();
    reader.setIgnoreComments(true);

    try {
      xmlDoc = reader.read(xmlDocName);
    }
    catch (DocumentException e) {
      System.err.println("Failed to parse the xml file, " + xmlDocName+", due to:\n");
      e.printStackTrace();
    }
  }

  /**
   * grabs a list of elements according the the xpath query
   * @param query xpath query
   * @return
   */
  public List grabElements(String query){
    return xmlDoc.selectNodes(query);
  }

  /**
   * grabs a list of element pathes according to the xpath query
   * @param query String xpath query
   * @return List of element pathes according to the xpath query
   */
  public List grabElementPathes(String query) {
      List elements = grabElements(query);
      List path = new ArrayList();
      for(int i=0; i<elements.size(); ++i) {
          Node node = (Node)elements.get(i);
          path.add(node.getPath());
      }
      return path;
  }

  public List grabValues(String query) {
      List elements = grabElements(query);
      List path = new ArrayList();
      for(int i=0; i<elements.size(); ++i) {
          Node node = (Node)elements.get(i);
          path.add(node.getText());
      }
      return path;

  }


  public String getProperty(String propPath) {

    Node propNode = xmlDoc.selectSingleNode(propPath);
    String value = propNode.getText();
    return value;
  }


}
